// Copyright (c) Open Enclave SDK contributors.
// Licensed under the MIT License.

#include <openenclave/tracee.h>
#include <stdio.h>
#include <string.h>

// Include the trusted log_callback header that is generated
// during the build. This file is generated by calling the
// sdk tool oeedger8r against the log_callback.edl file.
#include "log_callback_t.h"

void enclave_customized_log(
    void* context,
    oe_log_level_t level,
    uint64_t thread_id,
    const char* message)
{
    char modified_log[200];

    sprintf(
        modified_log,
        "E, %s, %llx, %s",
        oe_log_level_strings[level],
        thread_id,
        message);

    /*
     * Add logic here to modify the log message, to obscure enclave logs from
     * the host. The context might be used, for example, to transfer a
     * shared/public/private secret, that may be used to encrypt the log
     * message.
     *
     * NOTE: Do not use operations based on OE's host filesystem support
     * in this function, they do not work consistently. If used, the
     * logs generated after oe_terminate_enclave() cause the program to crash
     * with a segmentation fault (issue #4349).
     */
    OE_UNUSED(context);

    /* invoke ocall to copy enclave logs to file */
    oe_result_t result =
        host_transfer_logs_to_file(modified_log, strlen(modified_log));

    OE_UNUSED(result);
}

void enclave_set_log_callback()
{
    oe_enclave_log_set_callback(NULL, enclave_customized_log);
}

// This is the function that the host calls. It prints
// a message in the enclave before calling back out to
// the host to print a message from there too.
void enclave_hello()
{
    // Call back into the host
    oe_result_t result = host_hello();
    if (result != OE_OK)
    {
        fprintf(
            stderr,
            "Call to host_hello failed: result=%u (%s)\n",
            result,
            oe_result_str(result));
    }
}
